<?php

namespace Drupal\email_validator\Form;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Class EmailValidatorConfigForm.
 */
class EmailValidatorConfigForm extends ConfigFormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a settings controller.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The module handler.
   */
  public function __construct(
    ModuleHandlerInterface $moduleHandler,
    ConfigFactoryInterface $configFactory
  ) {
    parent::__construct($configFactory);
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'email_validator.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'email_validator_settings';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('config.factory')
    );
  }

  /**
   * Get a value from the retrieved form settings array.
   */
  public function getFormSettingsValue($form_settings, $form_id) {
    // If there are settings in the array and the form ID already has a setting,
    // return the saved setting for the form ID.
    if (!empty($form_settings) && isset($form_settings[$form_id])) {
      return $form_settings[$form_id];
    }
    // Default to false.
    else {
      return 0;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('email_validator.settings');

    // Url EVA.
    $link_eva = Link::fromTextAndUrl('https://e-va.io/', Url::fromUri('https://e-va.io/', [
      'attributes' => ['target' => '_blank'],
    ]))->toString();

    // - General Settings - API.
    $form['general'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('General Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];

    $form['general']['api_key'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Access Key'),
      '#default_value' => $config->get('general.api_key'),
      '#description'   => $this->t('Before you start, please make sure you have an account at @link_eva', ['@link_eva' => $link_eva]),
      '#maxlength'     => 255,
      '#required'      => TRUE,
      '#attributes'    => [
        'placeholder'  => 'PLACE_YOUR_KEY_HERE',
      ],
    ];

    // - Protection Settings.
    $form['protection'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Protection Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];
    $form['protection']['states_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disable EVA'),
      '#description' => $this->t('With this option all the states bellow will pass through EVA. Basically this means that our service is disabled.'),
      '#default_value' => $config->get('protection.states_all'),
    ];
    // Enabled States.
    $form['protection']['states'] = [
      '#type' => 'container',
      '#states' => [
        // Hide this fieldset when all states are accepted.
        'disabled' => [
          'input[name="states_all"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['protection']['states']['email_states'] = [
      '#type' => 'checkboxes',
      '#description' => $this->t('Select the states that you want to allow.'),
      '#default_value' => $config->get('protection.email_states'),
      '#options' => [
        0 => $this->t('Safe'),
        1 => $this->t('Unknown'),
        2 => $this->t('Invalid'),
        3 => $this->t('Risky'),
      ],
    ];

    // - Forms Configuration.
    $form['configuration'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Form Settings'),
      '#description' => $this->t("Check the boxes next to individual forms on which you'd like EVA validation be enabled."),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
    ];
    $form['configuration']['protect_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Validate all forms with EVA'),
      '#description' => $this->t('Enable EVA for ALL forms on this site.'),
      '#default_value' => $config->get('configuration.protect_all'),
    ];
    // Forms Configuration - Enabled forms.
    $form_settings = $config->get('configuration.forms');
    $form['configuration']['forms'] = [
      '#type' => 'container',
      '#states' => [
        // Hide this fieldset when all forms are protected.
        'invisible' => [
          'input[name="configuration[protect_all]"]' => ['checked' => TRUE],
        ],
      ],
      'label' => [
        '#type' => 'html_tag',
        '#type' => 'h5',
        '#value' => $this->t('User Forms'),
      ],
    ];
    // User register form.
    $form['configuration']['forms']['user_register_form'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('User Registration form'),
      '#default_value' => $this->getFormSettingsValue($form_settings, 'user_register_form'),
    ];

    // - Log Settings.
    $form['logs'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Log Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];
    $form['logs']['wrong_emails'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log rejected emails'),
      '#description' => $this->t('Report information about rejected emails to the log.'),
      '#default_value' => $config->get('logs.wrong_emails'),
    ];
    // Replace the description with a link if dblog.module is enabled.
    if ($this->moduleHandler->moduleExists('dblog')) {
      $form['logs']['wrong_emails']['#description'] = $this->t('Report information about rejected emails to the <a href=":dblog">log</a>.', [
        ':dblog' => Url::fromRoute('dblog.overview')->toString(),
      ]);
    }

    $form['logs']['system_down'] = [
      '#type' => 'radios',
      '#title' => $this->t("When EVA don't have credits or has no answers from the API"),
      '#description' => $this->t('Our services provides you 99.9% of uptime, however some connections problems could happen or even your credits may be empty.'),
      '#options' => [
        0 => $this->t('Reject all the validations'),
        1 => $this->t('Bypass all the validations'),
      ],
      '#default_value' => $config->get('logs.system_down'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('email_validator.settings')
      ->set('general.api_key', $form_state->getValue('api_key'))
      ->set('configuration.protect_all', $form_state->getValue('protect_all'))
      ->set('protection.states_all', $form_state->getValue('states_all'))
      ->set('protection.email_states', $form_state->getValue('email_states'))
      ->set('logs.wrong_emails', $form_state->getValue('wrong_emails'))
      ->set('logs.system_down', $form_state->getValue('system_down'));
    // Save the forms from $form_state into a 'configuration' array.
    $config->set('configuration', $form_state->getValue('configuration'));
    $config->save();
  }

}
